use palette::Oklch;
use serde::Serialize;
use wasm_bindgen::prelude::wasm_bindgen;

use super::{ColorDifference, Differ};

#[derive(Debug, Clone, Copy, Serialize)]
#[wasm_bindgen]
pub struct OklchDifference {
    pub hue: Differ,
    pub chroma: Differ,
    pub lightness: Differ,
}

impl ColorDifference for Oklch {
    type Difference = OklchDifference;

    fn difference(&self, other: &Self) -> Self::Difference {
        let hue = self.hue.into_positive_degrees() - other.hue.into_positive_degrees();
        let chroma = self.chroma - other.chroma;
        let lightness = self.l - other.l;

        OklchDifference {
            hue: Differ {
                delta: hue,
                percent: hue / self.hue.into_positive_degrees(),
            },
            chroma: Differ {
                delta: chroma,
                percent: chroma / self.chroma,
            },
            lightness: Differ {
                delta: lightness,
                percent: lightness / self.l,
            },
        }
    }
}
